function [x,info] = nmhp(M,q,x0,options)

%%
% [x] = nmhp(M,q,x0,options)
%
% Nmvar solves the linear complementarity problem in the form
%
%    0 <= x _|_ (M*x+q) >= 0,
%
% meaning that a vector x is sought such that the components of x and
% M*x+q are nonnegative and x'*(M*x+q) = 0. It is supposed that M is a
% real matrix of order n, that q is a real vector of length n, x0 is an
% optional initial vector for the iterative method, and options is an
% optional structure of options.
%
% The methods implemented in NMHP are various variants of the Newton-min
% algorithm. One uses as merit function the map defined at x in Rn by
%
%    Theta(x) = |min(x,M*x+q)|.
%
% Various options allow the user to tune the solver:
% . options.abstol (not compatible with 'options.reltol'): an iterate x
%     is considered to be a solution when the value Theta(x) is less
%     than options.abstol;
% . options.direction: type of search direction
%     = 0: B-Newton-min,
%     = 1: Newton-min (several options, including Harker-Pang, not yet
%          implemented),
%     = 2: Newton-min-var,
%     = 3: Newton-min-hybrid;
% . options.dxmin: resolution in x; if the increment to x is less than
%     options.dxmin in the sup-norm, the algorithm considers that it
%     cannot do better and stops; default value is 1.e-10;
% . options.fout: output channel for the outputs (0 for the screen, fout
%     = fopen('res.txt','w') for writing on the file 'res.txt');
% . options.itermax: maximum number of iterations allowed;
% . options.linesearch:
%     = 0: use the special stepsize specified by
%          options.linesearch_special;
%     = 1: with linesearch,
%     > 1: nonmonotone linesearch in which the merit function is only
%          forced to decrease with respect to its values taken at the
%          last options.linesearch iterates (not yet implemented);
% . options.linesearch_interpol specifies the way of determining the
%     stepsize in case options.linesearch > 0;
%     = false: the new trial stepsize is half the previous inapropriate
%          one,
%     = true: the new trial stepsize is obtained by interpolation using
%          the values of the least-square merit function at the current
%          iterate and at the last unsuccessful point, as well as the
%          slope at the least-square function at the current iterate
%          along the descent durection (should be verified);
% . options.linesearch_special specifies the way of determining the
%     stepsize in case options.linesearch = 0;
%     = 0: unit stepsize (no linesearch),
%     = 1: exact stepsize (find the global minimizer of the least-square
%          merit function along the search direction (not yet
%          implemented),
%     = 2: Harker-Pang stepsize (not yet implemented);
% . options.reltol (not compatible with 'options.abstol'): an iterate x
%     is considered to be a solution when the value Theta(x) is less
%     than options.reltol*Theta(x0);
% . options.verb: verbosity level of the output
%     = 0: no output; works silently,
%     = 1: one line per iteration,
%     = 2: one paragraph per iteration, including linesearch
%          information,
%     = 3: more output, requiring extra-computation, not essemtial for
%          finding a solution.
%
% On return
% . x: solution to the LCP,
% . info.iter: number of iterations;
% . info.flag: diagonis
%     = 0: solution found,
%     = 1: an input argument is wrong,
%     = 2: max number of iterations reached,
%     = 3: stop on dxmin,
%     = 4: failure of te Harker-Pang technique,
%     = 5: failure with Quadprog,
%     = 6: positive slope of the least-square function,
%     = 7: too many linesearch trials;
% . info.qpsolve: number of QP solve;

% Version 1.1, April 2019.

%-----------------------------------------------------------------------
%
% Author: Jean Charles Gilbert, INRIA.
% Copyright 2018, 2019, INRIA.
%
% NMHP is distributed under the terms of the Q Public License version
% 1.0.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Q Public
% License version 1.0 for more details.
%
% You should have received a copy of the Q Public License version 1.0
% along with this program.  If not, see
% <http://doc.trolltech.com/3.0/license.html>.
%
%-----------------------------------------------------------------------

%------------------------
% Parameters
%------------------------

  qp_options = optimset ('Display'      , 'off', ...
                         'Diagnostics'  , 'off', ...
                         'LargeScale'   , 'off', ...
                         'MaxIter'      , 1000, ...
                         'Algorithm'    , 'interior-point-convex');
%                        'Algorithm'    , 'active-set');

%------------------------
% Initialization
%------------------------

  tic_init = tic;	% Launch the timer

  [values] = nmhp_values ();	% Set the constant values used in NMHP
  info.flag = values.success;

  bs_tol = 1.e-12;	% tolerance for distinguishing equal break-stepsizes
  bs_cc  = 0.5;		% convex combinaison in (0,1) of the first two break-stepsizes to take when options.method == values.method_hpe

  %------------------------
  % Check input arguments
  %------------------------

  if nargin < 4, options = struct();
    if nargin < 3, x0 = [];
      if nargin < 2
        fprintf('\n### nmhp: the first 2 arguments are required\n\n');
        x = x0;
        info.flag = values.fail_on_argument;
        info.timer(values.cput_total) = toc(tic_init);
        return
      end
    end
  end

  q = q(:);
  x = x0(:);
  infos = [];

  %------------------------
  % Decode options
  %------------------------

  if ~exist('options') | isempty(options); options = struct(); end

  [M,q,x,y,options,info] = nmhp_prelim(M,q,x,options,values,info);
  if info.flag; return; end

  n = length(q);

  fout = options.fout;
  verb = options.verb;

%------------------------
% Loop
%------------------------

  info.qpsolve = 0;
  iter         = 0;

  A  = [];
  I  = [];
  Ep = [];
  Em = [];

  info.count_nonempty_e  = 0;
  info.count_nonempty_em = 0;

  if verb == 1
    fprintf (fout,'\n%s\niter  |min(x,M*x+q)|  |A0|  |I0|  |Ep|  |Em|',values.dline);
    if (options.direction == 2) | (options.direction == 3); fprintf (fout,'  QP'); end
    if options.linesearch; fprintf (fout,'   stepsize'); end
  end

  tol_merit = 0.5*options.abstol*options.abstol;	% options.abstol has been defined in nmhp_prelim

  while true

    %------------------------
    % Optimality
    %------------------------

    y = M*x+q;
    F = min(x,y);
    merit = 0.5*(F'*F);
    if merit <= tol_merit
      info.iter = iter;
      info.flag = values.success;
      break
    end

    %------------------------
    % Iteration counter
    %------------------------

    if iter >= options.itermax
      info.iter = iter;
      info.flag = values.stop_on_itermax;
      break
    end
    iter = iter+1;

    %------------------------
    % Print
    %------------------------

    if verb
      if verb == 1
        fprintf (fout,'\n%4i  %14.8e',iter,sqrt(2*merit));
      elseif verb > 1
        fprintf (fout,'\n%s',values.dline);
        fprintf (fout,'\niter = %0i, |min(x,M*x+q)| = %12.5e\n',iter,sqrt(2*merit));
        if verb > 3
          fprintf('\nx ='); fprintf(' %12.5e',x);
          fprintf('\ny ='); fprintf(' %12.5e',y);
        end
      end
    end

    %------------------------
    % Index sets
    %------------------------

    if options.method == values.method_els
      eps_active  = options.dxmin;
    else
      eps_active  = 0;
    end

    Aold  = A;
    Iold  = I;
    Epold = Ep;
    Emold = Em;

    E  = find(abs(x-y) <= eps_active);
    Ep = intersect(E,E(find(x(E)>=0)));		% intersection should be useless here
    Em = setdiff(E,Ep);
    I  = find(x > y+eps_active);
    Az = setdiff([1:n],union(E,I));
    A  = union(Az,Ep);

    if length(E)  ~= 0; info.count_nonempty_e  = info.count_nonempty_e +1; end
    if length(Em) ~= 0; info.count_nonempty_em = info.count_nonempty_em+1; end

    if verb == 1
      fprintf (fout,'%6i%6i%6i%6i',length(Az),length(I),length(Ep),length(Em));
    elseif verb > 1
      fprintf (fout,'\n|A0| = %0i, |I0| = %0i, |Ep| = %0i, |Em| = %0i\n',length(Az),length(I),length(Ep),length(Em));
    end

    if verb > 2
      Sm = setdiff(Aold,A);
      Sp = setdiff(A,Aold);
      if ~isempty(Sm) | ~isempty(Sp)
        fprintf (fout,'. Change in A:');
        if ~isempty(Sm); fprintf (fout,' -%0i',Sm); end
        if ~isempty(Sp); fprintf (fout,' +%0i',Sp); end
        fprintf (fout,'\n');
      end
      Sm = setdiff(Iold,I);
      Sp = setdiff(I,Iold);
      if ~isempty(Sm) | ~isempty(Sp)
        fprintf (fout,'. Change in I:');
        if ~isempty(Sm); fprintf (fout,' -%0i',Sm); end
        if ~isempty(Sp); fprintf (fout,' +%0i',Sp); end
        fprintf (fout,'\n');
      end
      Sm = setdiff(Epold,Ep);
      Sp = setdiff(Ep,Epold);
      if ~isempty(Sm) | ~isempty(Sp)
        fprintf (fout,'. Change in Ep:');
        if ~isempty(Sm); fprintf (fout,' -%0i',Sm); end
        if ~isempty(Sp); fprintf (fout,' +%0i',Sp); end
        fprintf (fout,'\n');
      end
      Sm = setdiff(Emold,Em);
      Sp = setdiff(Em,Emold);
      if ~isempty(Sm) | ~isempty(Sp)
        fprintf (fout,'. Change in Em:');
        if ~isempty(Sm); fprintf (fout,' -%0i',Sm); end
        if ~isempty(Sp); fprintf (fout,' +%0i',Sp); end
        fprintf (fout,'\n');
      end
      fprintf (fout,'. A0:'); fprintf (fout,' %0i',Az); fprintf (fout,'\n');
      fprintf (fout,'. I0:'); fprintf (fout,' %0i',I); fprintf (fout,'\n');
    end

    %------------------------
    % Direction
    %------------------------

    [d,info] = nmhp_direction (M,x,y,Az,A,I,E,Em,merit,qp_options,options,values,info);

    if isempty(d); break; end

% % The numerator is not constant
% 
% if options.linesearch_special == 3
%   num = M*x-ones(n,1)-x;
%   num = num(I);
%   den = M(I,I)\ones(length(I),1);
%   quo = num./den;
%   fprintf('\n\n i        x           M*x-e-x      fraction     bound        MII\\e     (M*x-x-e)./(M\\e)');
%   fprintf('\n^^^^  ^^^^^^^^^^^^^  ^^^^^^^^^^^^^  ^^^^^^^^^^  ^^^^^^^^^  ^^^^^^^^^^^^^  ^^^^^^^^^^^^^^');
%   for i = 2:length(I)
%     fprintf('\n%4i  %13.6e  %13.6e',I(i),x(I(i)),num(i));
%     if i+2 > length(I)
%       fprintf('                       ');
%     else
%       fprintf('  %10.4e  %9.3e',num(i)/num(i+2),(2*(n-I(i))+1)/(2*(n-I(i)-2)+1));
%     end
%     fprintf('  %13.6e  %14.7e',den(i),quo(i));
%   end
% end

    %------------------------
    % Break-stepsizes
    %------------------------

    ymx  = y-x;
    Md   = M*d;
    dmMd = d-Md;
    Ind  = find(dmMd~=0);
    alphab = ymx(Ind)./dmMd(Ind);
    alphab = sort(alphab(find((alphab>0)&(alphab<1))));	% break stepsizes in (0,1)
    % Remove too close break-stepsizes
    if bs_tol ~=0
      iref = 1;
      if length(alphab)
        for i=2:length(alphab)
          if alphab(i) > alphab(iref)+bs_tol
            iref = iref+1;
            alphab(iref) = alphab(i);
          end
        end
        alphab = alphab(1:iref);
      end
    end

    alphab1 = 0;
    alphab2 = inf;
    nalphab = length(alphab);
    if nalphab
      alphab1 = alphab(1);
      if nalphab > 1
        alphab2 = alphab(2);
      end
    end

    if verb > 1
      if nalphab
        fprintf (fout,'\n\nBreak-stepsize');
        if nalphab > 1; fprintf (fout,'s'); end
        fprintf (fout,' %11.5e',alphab);
      else
        fprintf (fout,'\n\nNo break-stepsize');
      end
    end

    %------------------------
    % Stepsize
    %------------------------

    if options.method == values.method_hp		% Harker and Pang

      if nalphab & (alphab1 < 1)

        [x,y,info] = nmhp_ls (M,q,x,d,merit,alphab,alphab1,alphab1+options.epshp,options,values,info);
	if info.flag; break; end

      else

        x = x+d;
        y = M*x+q;

      end

    elseif options.method == values.method_hpe		% extended Harker and Pang: alpha = (alphab1+alphab2)/2
    
      if nalphab > 1
        alpha = (1-bs_cc)*alphab1+bs_cc*alphab2;
      else
        alpha = 1;
      end
      if verb; fprintf (fout,'\n\nStepsize = %21.15e',alpha); end
      x = x + alpha*d;
      y = M*x+q;

    elseif options.method == values.method_els		% exact linesearch

        [x,y,info] = nmhp_els (M,q,x,d,merit,alphab,0,1,options,values,info);
	if info.flag; break; end

    end

  end

  %------------------------
  % Final printings
  %------------------------

  if verb < 0
    if info.flag == values.fail_on_recursivity_before
      fprintf (fout,'\n\n### Recursivity cannot be carried on since |E| = n = %i\n',n);
    end
  end

  if verb == 1
    fprintf (fout,'\n%4i  %14.8e',iter+1,sqrt(2*merit));
  end

  if verb > 0

    fprintf(fout,'\n%s',values.dline);

    switch info.flag
      case values.success
        fprintf (fout,'\nSolution found\n%s\nNumber of iterations = %0i\n|min(x,M*x+q)| = %11.5e',values.dline,info.iter,sqrt(2*merit));
      case values.stop_on_itermax
        fprintf (fout,'\n### Too many iterations\n%s',values.dline);
      case values.stop_on_dxmin
        fprintf (fout,'\n%s',values.dline);
%     case values.stop_on_epshp
%       fprintf (fout,'\n### Failure of the Harker-Pang technique\n%s',values.dline);
      case values.fail_with_quadprog
        % print already done
      case values.fail_on_recursivity_after
        fprintf(fout,'\n### Recursive call to nmhp failled, flag on output = %i\n%s',nmhp_info.flag,values.dline);
      case values.fail_on_positive_slope
        fprintf (fout,'\n### Positive slope of the least-square function\n%s',values.dline);
      case values.stop_on_iterlsmax
        fprintf (fout,'\n### too many linesearch trials (%i)\n%s',iter_ls_max,values.dline);
    end

    if info.flag == values.success

      Aold  = A;
      Iold  = I;
      Epold = Ep;
      Emold = Em;

      E  = find(abs(x-y) <= eps_active);
      Ep = intersect(E,E(find(x(E)>=-eps_active)));
      Em = setdiff(E,Ep);
      I  = find(x > y+eps_active);
      Az = setdiff([1:n],union(E,I));
      A  = union(Az,Ep);

      if verb

        fprintf (fout,'\n|A0| = %0i, |I0| = %0i, |Ep| = %0i, |Em| = %0i',length(Az),length(I),length(Ep),length(Em));

        if verb > 2
          Sm = setdiff(Aold,A);
          Sp = setdiff(A,Aold);
          if ~isempty(Sm) | ~isempty(Sp)
            fprintf (fout,'\n. Change in A:');
            if ~isempty(Sm); fprintf (fout,' -%0i',Sm); end
            if ~isempty(Sp); fprintf (fout,' +%0i',Sp); end
          end
          Sm = setdiff(Iold,I);
          Sp = setdiff(I,Iold);
          if ~isempty(Sm) | ~isempty(Sp)
            fprintf (fout,'\n. Change in I:');
            if ~isempty(Sm); fprintf (fout,' -%0i',Sm); end
            if ~isempty(Sp); fprintf (fout,' +%0i',Sp); end
          end
          Sm = setdiff(Epold,Ep);
          Sp = setdiff(Ep,Epold);
          if ~isempty(Sm) | ~isempty(Sp)
            fprintf (fout,'\n. Change in Ep:');
            if ~isempty(Sm); fprintf (fout,' -%0i',Sm); end
            if ~isempty(Sp); fprintf (fout,' +%0i',Sp); end
          end
          Sm = setdiff(Emold,Em);
          Sp = setdiff(Em,Emold);
          if ~isempty(Sm) | ~isempty(Sp)
            fprintf (fout,'\n. Change in Em:');
            if ~isempty(Sm); fprintf (fout,' -%0i',Sm); end
            if ~isempty(Sp); fprintf (fout,' +%0i',Sp); end
          end
          if ~isempty(Az); fprintf (fout,'\n. A0:'); fprintf (fout,' %0i',Az); end;
          if ~isempty(I);  fprintf (fout,'\n. I0:'); fprintf (fout,' %0i',I);  end;
          if ~isempty(E);  fprintf (fout,'\n. E:');  fprintf (fout,' %0i',E);  end;
        end

        fprintf(fout,'\nNumber of iterations with\n. nonempty E (%i)\n. nonempty Em (%i)\n. QP solve (%i)', ...
          info.count_nonempty_e,info.count_nonempty_em,info.qpsolve);
        fprintf(fout,'\nCPU time = %8.2e sec',toc(tic_init));

      end

    end

    fprintf(fout,'\n%s\n',values.eline);

  end

  return
