function [d,info] = nmhp_direction (M,x,y,Az,A,I,E,Em,merit,qp_options,options,values,info)

%%
% [d,info] = nmhp_direction (M,x,y,Az,A,I,E,Em,merit,qp_options,options,values,info)
%
% Computes a direction d at x for the Newton-min algorithm, which solves
% the linear complementarity problem 0 <= x _|_ y >= 0, with y = M*x+q.
% The following index sets are useful
%
%   Az = {i: x(i) < y(i)-eps_active},
%   I  = {i: x(i) > y(i)+eps_active},
%   E  = [1:n] \ {Az union I},
%   Ep = {i in E: x(i) >= 0}
%   Em = setdiff(E,Ep);
%
% where eps_active > 0 is a small number. The type of direction is
% determined by 'options.direction', whose value can be
%
%   = 0: computes the B-Newton-min direction, that is the one for which
%        another LCP is solved when there are indices i with E;
%   = 2: computes the Newton-min-var direction, that is the one for
%        which a convex QP is solved when there are indices in Em;
%   = 3: computes the Newton-min-hybrid direction, that is similar to
%        the Newton-min-var direction, except that it is first checked
%        whether setting d(Em) = -x(Em) does yield a direction of
%        sufficient decrease for the least-square merit function; if so,
%        it is taken (hence a single linear system is solved), otherwise
%        the Newton-min-var direction is computed (hence a convex QP of
%        size |Em| is solved).
%
% If the direction cannot be computed, d is empty on output and the
% reason why this occur is given in info.flag (for the actual values of
% the variables values.*, see the file 'nmhp_values.m'):
%
%   = values.fail_with_quadprog
%   = values.fail_with_linear_solve (probably due to the degeneracy of
%        M)

% Version 1.1, April 2019.

%-----------------------------------------------------------------------
%
% Author: Jean Charles Gilbert, INRIA.
% Copyright 2018, 2019, INRIA.
%
% NMHP is distributed under the terms of the Q Public License version
% 1.0.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Q Public
% License version 1.0 for more details.
%
% You should have received a copy of the Q Public License version 1.0
% along with this program.  If not, see
% <http://doc.trolltech.com/3.0/license.html>.
%
%-----------------------------------------------------------------------

%------------------------
% Parameter
%------------------------

  frac2 = 0.25;	% in (0,2), this fraction of 2 is used to see whether Theta has a negative slope along the NM direction

%------------------------
% Initialization
%------------------------

  d    = [];
  n    = length(x);
  nI   = length(I);
  verb = options.verb;
  fout = options.fout;

%------------------------
% factorization of MII = M(I,I)
%------------------------

  if nI
    MII = M(I,I);
    [L,U,P]=lu(MII);
    if (prod(diag(L)) == 0) | (prod(diag(U)) == 0)
      fprintf (fout,'\n### nmhp_direction: the matrix M is probably degenerate, since M(I,I) is');
      fprintf (fout,'\n    likely to be singular (here, I = {%i',I(1));
      fprintf (fout,',%i',I(2:end));
      fprintf (fout,'})\n');
      d = [];
      info.flag = values.fail_with_linear_solve;
      return
    end
  end

  if verb > 1; fprintf (fout,'\nDirection computation'); end

%------------------------
% B-Newton-min direction
%------------------------

  if options.direction == 0

    if isempty(E)

      if nI
        if isempty(Az)
          d = -M\y;
          if verb > 1; fprintf (fout,'\n. a single LS with %i variable(s) to solve',n); end
        else
          d     = zeros(n,1);
          d(Az) = -x(Az);
          d(I)  = U\(L\(P*(M(I,Az)*x(Az)-y(I))));			% d(I)  = MII\(M(I,Az)*x(Az)-y(I));
          if verb > 1; fprintf (fout,'\n. a single LS with %i variable(s) to solve',nI); end
        end
      else
        d = -x;
        if verb > 1; fprintf (fout,'\n. no LS or LCP to solve'); end
      end

    else

      nE = length(E);

      ME  = M(E,E);
      qE  = y(E);
      if ~isempty(Az)
        qE  = qE-M(E,Az)*x(Az);
      end
      if nI
        ME  = ME-M(E,I)*(U\(L\(P*M(I,E))));			% ME  = ME-M(E,I)*(MII\M(I,E));
        qE  = qE-ME*x(E);
        if isempty(Az)
          qE = qE-M(E,I)*(U\(L\(P*y(I))));			% qE = qE-M(E,I)*(MII\(y(I)));
        else
          qE = qE-M(E,I)*(U\(L\(P*(y(I)-M(I,Az)*x(Az)))));	% qE = qE-M(E,I)*(MII\(y(I)-M(I,Az)*x(Az)));
        end
      else
        qE  = qE-ME*x(E);
      end

      xEnew = lemke(ME,qE,x(E));
%     xEnew = lcpsolve(-ME,-qE,zeros(nE,1),1.e20*ones(nE,1),x(E));	% happened to give better results than 'lemke' on fathi-2048-nmb-s2
%     xEnew = lcplemke(-ME,-qE,zeros(nE,1),1.e20*ones(nE,1),x(E));	% less precise than lcpsolve for large dimensions (use lemke instead)

      d    = zeros(n,1);
      dE   = xEnew-x(E);
      d(E) = dE;
      if ~isempty(Az); d(Az) = -x(Az); end
      if nI
        v = y(I)+M(I,E)*dE;
        if ~isempty(Az); v = v - M(I,Az)*x(Az); end
        d(I) = -(U\(L\(P*v)));					% d(I) = -MII\v;
      end

      if verb > 1
        Md = M*d;
        compl = max(abs((x(E)+d(E)).*(y(E)+Md(E))));
        fprintf (fout,'\n. an LCP with %i variable(s) is solved using ''lemke''',nE);
%       fprintf (fout,'\n. an LCP with %i variable(s) is solved using 'lcpsolve'',nE);
%       fprintf (fout,'\nDirection computation with |E| = %i, using lcplemke',nE);
        fprintf (fout,'\n. positivity: (x+d)(E) >= %8.2e , (y+M*d)(E) >= %8.2e',min(xEnew),min(ME*xEnew+qE));
        fprintf (fout,'\n. complementarity in E components = %8.2e',compl);
        fprintf (fout,'\n. |d(E)| = %8.2e, |d| = %8.2e',norm(dE),norm(d));
      end

    end

%------------------------
% Newton-min direction
%------------------------

  elseif options.direction == 1

    A = union(Az,E);
    d(A) = -x(A);
    if nI
      rhs = -y(I);
      if ~isempty(A); rhs = rhs+M(I,A)*x(A); end
      d(I) = U\(L\(P*rhs));			% d(I)  = MII\(M(I,A)*x(A)-y(I));
    end
    if verb > 1; fprintf (fout,'\n. a single LS with %i variable(s) to solve',nI); end
    d = d(:);

%------------------------
% Newton-min-var direction
%------------------------

  elseif options.direction == 2

    if nI	% vector v = MII \ (y(I) - M(I,A)*x(A) )
      if isempty(A)
        v = U\(L\(P*y(I)));			% v = MII\y(I)
      else
        v = U\(L\(P*(y(I)-M(I,A)*x(A))));	% v = MII\(y(I)-M(I,A)*x(A))
      end
    else
      v = [];
    end

    if isempty(Em)	% no QP to solve

      if verb
        if verb == 1
          fprintf('    ');
        else
          fprintf (fout,'\n. no QP to solve');
        end
      end

      if isempty(I)
        d = -x;
      else
        if isempty(A)
          d = -v;
        else
          d = zeros(n,1);
          d(A) = -x(A);
          d(I) = -v;
        end
      end

    else		% a QP to solve in the unknown d(Em)

      nEm = length(Em);

      if verb == 1; fprintf(' %3i',nEm); end

      Id = eye(nEm,nEm);

      if isempty(I)
        H = Id;
        g = zeros(nEm,1);
      else
        MIImMIE = U\(L\(P*M(I,Em)));		% MIImMIE = MII\M(I,Em);
        H = MIImMIE'*MIImMIE+Id;
        g = MIImMIE'*v;
      end

      if isempty(I)
        Ms = M(Em,Em);
      else
        Ms = M(Em,Em) - M(Em,I)*MIImMIE;	% Schur complement
      end
      Ai = - [Id;
              Ms];

      yEm = y(Em);
      bi = yEm;
      if ~isempty(A)
        bi = bi - M(Em,A)*x(A);
      end
      if nI
        bi = bi - M(Em,I)*v;
      end
      xEm = x(Em);
      bi = [xEm;
            bi];

      [dEm,~,info_qp] = quadprog(H,g,Ai,bi,[],[],[],[],[],qp_options);

      info.qpsolve = info.qpsolve+1;

      if info_qp ~= 1
        info.iter = iter;
        info.flag = values.fail_with_quadprog;
        if verb > 1
          fprintf(fout,'\n%s',values.dline);
          fprintf(fout,'\nFailure with Quadprog (');
          switch info_qp
            case 0
              fprintf (fout,'too many iterations)\n');
            case -2
              fprintf (fout,'infeasible problem)\n');
            case -3
              fprintf (fout,'unbounded problem)\n');
            otherwise
              fprintf (fout,'exit flag = %0i)\n',info_qp);
          end
        end
        return
      end

      d = zeros(n,1);
      d(Em) = dEm;
      if ~isempty(A)
        d(A) = -x(A);
      end
      if nI
        d(I) = - ( U\(L\(P*(M(I,Em)*dEm))) + v );			% d(I) = - ( MII\(M(I,Em)*dEm) + v );
      end

      Md = M*d;
      compl = max(abs((xEm+dEm).*(yEm+Md(Em))));
      if verb > 1
        fprintf (fout,'\n. a QP to solve in %i variable',nEm);
	if nEm > 1; fprintf('s'); end
        fprintf (fout,'\n. complementarity in Em components = %8.2e',compl);
      end

    end

%------------------------
% Newton-min-hybrid direction
%------------------------

  elseif options.direction == 3

    if nI	% vector v = MII \ ( y(I) - M(I,A)*x(A) )
      if isempty(A)
        v = U\(L\(P*y(I)));			% v = MII\y(I)
      else
        v = U\(L\(P*(y(I)-M(I,A)*x(A))));	% v = MII\(y(I)-M(I,A)*x(A))
      end
    else
      v = [];
    end

    if isempty(Em)	% no QP to solve

      if verb
        if verb == 1
          fprintf('    ');
        else
          fprintf (fout,'\n. no QP to solve');
        end
      end

      if nI
        if isempty(A)
          d = -v;
        else
          d    = zeros(n,1);
          d(A) = -x(A);
          d(I) = -v;
        end
      else
        d = -x;
      end

    else

      % try first by putting Em in A (hence no QP to solve in that cae)

      AA = union(A,Em);

      if nI
        d     = zeros(n,1);
        d(AA) = -x(AA);
        d(I)  = U\(L\(P*(M(I,AA)*x(AA)-y(I))));		% d(I)  = MII\(M(I,AA)*x(AA)-y(I))
      else
        d = -x;
      end

      slopeincrease = x(E)'*min(x(E)+d(E),y(E)+M(E,:)*d);
      if slopeincrease <= (2-frac2)*merit
        if verb
          if verb == 1
            fprintf('    ');
          else
            fprintf (fout,'\n. no QP to solve: slope %22.15e',-2*merit);
            fprintf (fout,'\n                      + %22.15e',slopeincrease);
            fprintf (fout,'\n                      = %22.15e',-2*merit+slopeincrease);
          end
        end
        return
      end

      % it is not a direction of sufficient decrease, hence solve a QP to find a descent direction

      nEm = length(Em);

      if verb == 1; fprintf(' %3i',nEm); end

      Id = eye(nEm,nEm);

      if nI
        MIImMIE = U\(L\(P*M(I,Em)));			% MIImMIE = MII\M(I,Em)
        H = MIImMIE'*MIImMIE+Id;
        g = MIImMIE'*v;
      else
        H = Id;
        g = zeros(nEm,1);
      end

      if nI
        Ms = M(Em,Em) - M(Em,I)*MIImMIE;	% Schur complement
      else
        Ms = M(Em,Em);
      end
      Ai = - [Id;
              Ms];

      yEm = y(Em);
      bi = yEm;
      if ~isempty(A)
        bi = bi - M(Em,A)*x(A);
      end
      if nI
        bi = bi - M(Em,I)*v;
      end
      xEm = x(Em);
      bi = [xEm;
            bi];

      [dEm,~,info_qp] = quadprog(H,g,Ai,bi,[],[],[],[],[],qp_options);

      info.qpsolve = info.qpsolve+1;

      if info_qp ~= 1
        info.iter = iter;
        info.flag = values.fail_with_quadprog;
        if verb > 1
          fprintf(fout,'\n%s',values.dline);
          fprintf(fout,'\nFailure with Quadprog (');
          switch info_qp
            case 0
              fprintf (fout,'too many iterations)\n');
            case -2
              fprintf (fout,'infeasible problem)\n');
            case -3
              fprintf (fout,'unbounded problem)\n');
            otherwise
              fprintf (fout,'exit flag = %0i)\n',info_qp);
          end
        end
        return
      end

      d = zeros(n,1);
      d(Em) = dEm;
      if ~isempty(A)
        d(A) = -x(A);
      end
      if nI
        d(I) = - ( U\(L\(P*(M(I,Em)*dEm))) + v );			% d(I) = - ( MII\(M(I,Em)*dEm) + v );
      end

      Md = M*d;
      compl = max(abs((xEm+dEm).*(yEm+Md(Em))));
      if verb > 1; fprintf (fout,'\nComplementarity in Em components = %8.2e',compl); end

    end

  end

end
