function [x,y,info] = nmhp_els (M,q,x,d,merit,alphab,alphamin,alphamax,options,values,info);

%%
% [x,y,info] = nmhp_els (M,q,x,d,merit,alphab,alphamin,alphamax,options,values,info);
%
% Exact linesearch function. This function finds a stepsize in
% (alphamin, alphamax) from x along the direction d that minimise the
% function x -> norm(min(x,M*x+q))^2. Along d, this function is
% piecewise convex quadratic, and convex quadratic between the break
% stepsizes in 'alphab'.

% Version 1.1, April 2019.

%-----------------------------------------------------------------------
%
% Author: Jean Charles Gilbert, INRIA.
% Copyright 2018, 2019, INRIA.
%
% NMHP is distributed under the terms of the Q Public License version
% 1.0.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Q Public
% License version 1.0 for more details.
%
% You should have received a copy of the Q Public License version 1.0
% along with this program.  If not, see
% <http://doc.trolltech.com/3.0/license.html>.
%
%-----------------------------------------------------------------------

% Initialization

  y = [];

  n = length(q);

  verb = options.verb;
  fout = options.fout;

  dn = norm(d,inf);
  damin = options.dxmin/dn;

  Md = M*d;

  alphadiff = alphamax-alphamin;
  if alphadiff < damin
    if verb; fprintf (fout,'\n\n### Too small change in x (= %8.2e < dxmin = %8.2e)\n%s',alphadiff*dn,options.dxmin); end
    info.flag = values.stop_on_dxmin;
    return
  end

  slope = -2*merit;
  if slope >= 0
    info.flag = values.fail_on_positive_slope;
    return
  end
  if verb > 1
    fprintf (fout,'\n\nExact linesearch between %12.6e and %12.6e,  |d| = %12.6e,  damin = %12.6e',alphamin,alphamax,dn,damin);
    fprintf (fout,'\n     alpha0     |A0|  |E0|  |I0|     theta0       thetap0   | ');
    fprintf (fout,'   alpha1     |A1|  |E1|  |I1|     theta1       thetap1   |    alpha         theta');
  end

  alphab = alphab(find((alphab>=alphamin)&(alphab<=alphamax)));
  nalphab = length(alphab);
  if nalphab
    if alphamin < alphab(1)
      alphab = [alphamin;alphab];
      nalphab = nalphab+1;
    end
    if alphamax > alphab(nalphab)
      alphab = [alphab;alphamax];
      nalphab = nalphab+1;
    end
  else
    alphab  = [alphamin;alphamax];
    nalphab = 2;
  end

% Iterations

  alpha1 = alphab(1);
  x1 = x+alpha1*d;
  y1 = M*x1+q;

  A1 = find(x1<y1-damin);
  I1 = find(x1>y1+damin);
  E1 = setdiff([1:n],union(A1,I1));

  F1      = min(x1,y1);
  Fp1     = zeros(n,1);
  Fp1(A1) = d(A1);
  Fp1(E1) = max(d(E1),Md(E1));
  Fp1(I1) = Md(I1);
  theta1  = 0.5*(F1'*F1);

  alpha = zeros(nalphab-1,1);
  theta = zeros(nalphab-1,1);

  for i = 1:nalphab-1

    alpha0 = alpha1;
    theta0 = theta1;

    F0      = F1;
    Fp0     = Fp1;
    Fp0(E1) = min(d(E1),Md(E1));
    thetap0 = F0'*Fp0;

    if verb; fprintf (fout,'\n  %12.6e  %4i  %4i  %4i  %12.6e  %11.4e | ',alpha0,length(A1),length(E1),length(I1),theta0,thetap0); end

    alpha1 = alphab(i+1);

    x1 = x+alpha1*d;
    y1 = M*x1+q;

    A1 = find(x1<y1-damin);
    I1 = find(x1>y1+damin);
    E1 = setdiff([1:n],union(A1,I1));

    F1      = min(x1,y1);
    Fp1(A1) = d(A1);
    Fp1(E1) = max(d(E1),Md(E1));
    Fp1(I1) = Md(I1);
    theta1  = 0.5*(F1'*F1);
    thetap1 = F1'*Fp1;

    if verb; fprintf (fout,'%12.6e  %4i  %4i  %4i  %12.6e  %11.4e | ',alpha1,length(A1),length(E1),length(I1),theta1,thetap1); end

    % Compute the min in the interval [alpha0,alpha1]

    if (thetap0 <= 0) & (thetap1 <= 0)
      alpha(i) = alpha1;
      theta(i) = theta1;
    elseif (thetap0 >= 0) & (thetap1 >= 0)
      alpha(i) = alpha0;
      theta(i) = theta0;
    else
      alpha(i) = alpha0 - (alpha1-alpha0)*thetap0/(thetap1-thetap0);
      xx       = x + alpha(i)*d;
      yy       = M*xx+q;
      FF       = min(xx,yy);
      theta(i) = 0.5*(FF'*FF);
    end

    if verb; fprintf (fout,'%12.6e  %12.6e',alpha(i),theta(i)); end

  end

  [~,ind] = min(theta);
  a       = alpha(ind(1));
  x       = x+a*d;
  y       = M*x+q;

  if verb; fprintf (fout,'\n  Stepsize giving the minimum of Theta = %12.6e',a); end

  return
