function [x,y,info] = nmhp_ls (M,q,x,d,merit,alphab,alphamin,alphainit,options,values,info);

%%
% [x,y,info] = nmhp_ls (M,q,x,d,merit,alphab,alphamin,alphainit,options,values,info);
%
% Linesearch function. This function finds a point in (alphamin,
% alphainit), starting with alphainit and forcing the decrease of
% 0.5*norm(min(x,M*x+q))^2 at x in the direction d

% Version 1.1, April 2019.

%-----------------------------------------------------------------------
%
% Author: Jean Charles Gilbert, INRIA.
% Copyright 2018, 2019, INRIA.
%
% NMHP is distributed under the terms of the Q Public License version
% 1.0.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Q Public
% License version 1.0 for more details.
%
% You should have received a copy of the Q Public License version 1.0
% along with this program.  If not, see
% <http://doc.trolltech.com/3.0/license.html>.
%
%-----------------------------------------------------------------------

% Parameters

  iter_ls_max = 50;
  step_factor = 0.5;	% 1/pi;
  omega       = 1.e-4;

% Initialization

  y = [];

  verb = options.verb;
  fout = options.fout;

  dn = norm(d,inf);
  alpha_dxmin = options.dxmin/dn;

  alphadiff = alphainit-alphamin;
  if alphadiff < alpha_dxmin
    if verb; fprintf (fout,'\n\n### Too small change in x (= %8.2e < dxmin = %8.2e)\n%s',alphadiff*dn,options.dxmin); end
    info.flag = values.stop_on_dxmin;
    return
  end

  slope = -2*merit;
  if slope >= 0
    info.flag = values.fail_on_positive_slope;
    return
  end
  iter_ls = 0;
  if verb > 1
    fprintf (fout,'\n\nLinesearch\n  stepsize min = %12.6e, |d| = %12.6e, slope = %12.5e\n  iter    stepsize     merit decr    slope estim', ...
      alphamin,norm(d),slope);
  end

  alpha = alphainit;

% Iterations

  while true

    iter_ls = iter_ls+1;
    if iter_ls > iter_ls_max
      info.flag = values.stop_on_iterlsmax;
      break
    end

    xp = x+alpha*d;
    yp = M*xp+q;
    Fp = min(xp,yp);
    meritp = 0.5*Fp'*Fp;

    if verb > 1; fprintf (fout,'\n  %4i  %12.6e  %12.5e  %12.5e',iter_ls,alpha,meritp-merit,(meritp-merit)/alpha); end

    if meritp <= merit + omega*alpha*slope
      if verb == 1; fprintf (fout,'  %10.4e',alpha); end
      x = xp;
      y = yp;
      break;
    end

    while true
      alphadiff = step_factor*alphadiff;
      if alphadiff < alpha_dxmin
        if verb; fprintf (fout,'\n\n### Too small change in x (= %8.2e < dxmin = %8.2e)\n%s',alphadiff*dn,options.dxmin); end
        info.flag = values.stop_on_dxmin;
        break;
      end
      alpha = alphamin+alphadiff;
      if ~ismember(alpha,alphab); break; end
    end

  end
