function [M,q,x,y,options,info] = nmhp_prelim (M,q,x,options,values,info);

%%
% [M,q,x,y,options,info] = nmhp_prelim (M,q,x,options,values,info);
%
% This function realizes the following preliminary tasks:
% - set options (default values if absent, numeric values for
%   lexical options)
% - check the given options
% - get the dimensions
% - initial printings

% Version 1.1, April 2019.

%-----------------------------------------------------------------------
%
% Author: Jean Charles Gilbert, INRIA.
% Copyright 2018, 2019, INRIA.
%
% NMHP is distributed under the terms of the Q Public License version
% 1.0.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Q Public
% License version 1.0 for more details.
%
% You should have received a copy of the Q Public License version 1.0
% along with this program.  If not, see
% <http://doc.trolltech.com/3.0/license.html>.
%
%-----------------------------------------------------------------------

  y = [];

% Set fout and verb

  if ~isstruct(options)
    warning('nmhp:OptionsNotStruct','argument ''options'' is expected to be a structure\n');
    info.flag = values.fail_on_argument;
    return
  end
  if isfield(options,'fout')
    if isempty(options.fout) | (options.fout <= 0) | (fix(options.fout)-options.fout)
      if options.verb, fprintf('\n### nmhp: options.fout = %g not recongnized\n',options.fout); end
      info.flag = values.fail_on_argument;
      return
    end
  end
  if ~isfield(options,'fout')
    options.fout = 1;			% default output channel
  end
  fout = options.fout;

  if isfield(options,'verb')
    if isempty(options.verb)
      fprintf(fout,'\n### nmhp: options.verb must be nonnegative\n');
      info.flag = values.fail_on_argument;
      return
    end
  else
    options.verb = 1;			% default verbosity level
  end
  verb = options.verb;			% to reduce time access

  n = length(q);

% Check options fields

  possible_options = {'abstol', 'dxmin', 'epshp', 'fout', 'itermax', 'method', 'verb'};
  actual_options = fieldnames(options);
  for i=1:length(actual_options)
    if ~ismember(actual_options(i,:),possible_options)
      fprintf(fout,'\n### nmhp: field ''%s'' of structure ''options'' is not recognized\n',char(actual_options(i)));
      info.flag = values.fail_on_argument;
      return
    end
  end

% Set the some options, according to the chosen method

  if isfield(options,'method')
    if isempty(options.method) | (options.method < 0) | (options.method > 2)
      if verb; fprintf(fout,'\n### nmhp: options.method must be 0 (Harker and Pang method), 1 (extended Harker and Pang method) or 2 (exact LS)\n'); end
      info.flag = values.fail_on_argument;
      return
    end
  else
    options.method = 0;	% default: Harker and Pang
  end

  options.linesearch          = 0;	% 0: no LS, 1: LS, m>1: nonmonotone LS on the last m values of the merit function
  options.linesearch_interpol = false;

  if options.method == 0
    options.method = values.method_hp;	% Harker and Pang
    options.direction          = 1;	% 0: B-Newton-min, 1: Newton-min (several options, including Harker-Pang), 2: Newton-min-var, 3: Newton-min-hybrid
  elseif options.method == 1
    options.method = values.method_hpe;	% extended Harker and Pang
    options.direction          = 1;	% 0: B-Newton-min, 1: Newton-min (several options, including Harker-Pang), 2: Newton-min-var, 3: Newton-min-hybrid
  elseif options.method == 2	
    options.method = values.method_els;	% Exact linesearch
    options.direction          = 3;	% 0: B-Newton-min, 1: Newton-min (several options, including Harker-Pang), 2: Newton-min-var, 3: Newton-min-hybrid
  end

% Decode the other options

  if isfield(options,'itermax')
    if isempty(options.itermax) | (options.itermax <= 0)
      if verb; fprintf(fout,'\n### nmhp: options.itermax must be > 0\n'); end
      info.flag = values.fail_on_argument;
      return
    end
  else
    options.itermax = 1000;			% default maximal number of iterations
  end

  if isfield(options,'dxmin')
    if isempty(options.dxmin) | (options.dxmin < 0)
      if verb; fprintf(fout,'\n### nmhp: options.dxmin must be >= 0\n'); end
      info.flag = values.fail_on_argument;
      return
    end
  else
    options.dxmin = 1.e-11;			% default resolution in x
  end

  if isfield(options,'abstol') & isfield(options,'reltol')
    if verb; fprintf(fout,'\n### nmhp: at most one of the tolerances options.abstol and options.reltol must be specified\n'); end
    info.flag = values.fail_on_argument;
    return
  end
  if isfield(options,'abstol')
    if isempty(options.abstol) | (options.abstol <= 0)
      if verb; fprintf(fout,'\n### nmhp: options.abstol must be > 0\n'); end
      info.flag = values.fail_on_argument;
      return
    end
  elseif isfield(options,'reltol')
    if isempty(options.reltol) | (options.reltol <= 0) | (options.reltol > 1)
      if verb; fprintf(fout,'\n### nmhp: options.reltol must be in (0,1]\n'); end
      info.flag = values.fail_on_argument;
      return
    end
    if isempty(x)
      if verb; fprintf(fout,'\n### nmhp: x0 must be nonempty when the tolerance options.reltol is used\n'); end
      info.flag = values.fail_on_argument;
      return
    end
  else
    options.abstol = 1.e-8;			% default required relative precision
  end

  % Type of QP solving

  if isfield(options,'direction')
    if isempty(options.direction) | (fix(options.direction) ~= options.direction) | (options.direction < 0) | (options.direction > 3)
      if verb; fprintf(fout,'\n### nmhp: options.direction must be 0, 2, or 3\n'); end
      info.flag = values.fail_on_argument;
      return
    end
  else
    options.direction = 3;			% default: use the Newton-min-hybrid direction
  end

  % Type of linesearch

  if isfield(options,'linesearch')
    if isempty(options.linesearch) | (fix(options.linesearch) ~= options.linesearch) | (options.linesearch < 0) | (options.linesearch > 1)
      if verb; fprintf(fout,'\n### nmhp: ''options.linesearch'' must be 0 or 1\n'); end
      info.flag = values.fail_on_argument;
      return
    end
    if verb & (options.linesearch ~= 0) & isfield(options,'linesearch_special')
      fprintf(fout,'\n### nmhp: ''options.linesearch_special'' ignored since ''options.linesearch'' ~= 0\n');
    end
  else
    options.linesearch = 1;			% default Armijo linesearch
  end

  if isfield(options,'epshp')
    if options.epshp <= 0
      if verb; fprintf(fout,'\n### nmhp: ''options.epshp'' must be positive\n'); end
      info.flag = values.fail_on_argument;
      return
    end
  else
    options.epshp = 1.e-5;			% default Harker-Pang extra stepsize
  end

% Initial x, y, F, and options.abstol

  if isempty(x); x = zeros(n,1); end
  y = M*x+q;
  if ~isfield(options,'abstol')	% then, by the logic above, options.reltol is set
    options.abstol = options.reltol*abs(min(x,y));
  end

% Print

  if verb
    fprintf(fout,'\n%s',values.eline);
    fprintf (fout,'\nNMHP (LCP solver, version 1.1, April 2019)',n);
    fprintf (fout,'\n^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^',n);
    if issparse(M)
      fprintf (fout,'\n. Sparse data');
    else
      fprintf (fout,'\n. Dense data');
    end
    fprintf (fout,'\n. Number of variables          = %6i',n);
    fprintf (fout,'\n. Max iterations (itermax)     = %6i',options.itermax);
    if isfield(options,'abstol')
      fprintf (fout,'\n. Absolute tolerance (abstol)  =      %8.2e',options.abstol);
    else
      fprintf (fout,'\n. Relative tolerance (reltol)  =      %8.2e',options.reltol);
    end
    fprintf (fout,'\n. Resolution in x (dxmin)      =      %8.2e',options.dxmin);
    if options.direction == 0
      fprintf (fout,'\n. B-Newton-min direction');
    elseif options.direction == 1
      fprintf (fout,'\n. Newton-min direction');
    elseif options.direction == 2
      fprintf (fout,'\n. Newton-min-var direction');
    elseif options.direction == 3
      fprintf (fout,'\n. Newton-min-hybrid direction');
    end
    if options.method == values.method_hp
      fprintf (fout,'\n. Harker-Pang extra stepsize   =      %8.2e',options.epshp);
    elseif options.method == values.method_hpe
      fprintf (fout,'\n. Middle break-stepsize');
    elseif options.method == values.method_els
      fprintf (fout,'\n. Exact linesearch');
    end
  end

% Check that x is not on a kink

  if (options.method == values.method_hp) | (options.method == values.method_hpe)
    if ~isempty(find(x==y))
      fprintf (fout,'\n\n### For the chosen method, the initial x must not be on a kink of min(x,M*x+q),');
      fprintf (fout,'\n    meaning that, for y=M*x+q, there must hold x(i) ~= y(i) for all i in [1:n].\n\n');
      info.flag = -1;
    end
  end

% Exit

% if verb >= 2
%   fprintf(fout,'\n%s',values.dline);
% end

  return
